
structure MyBigNums :> BIGNUMS = struct

  val base = 1000
  val digits = 3

  type bignum = {neg:bool, coeffs:int list}

  fun unimplemented (s:string):'a =
    raise Fail (String.concat ["ERROR: Function ",s, " not implemented."])

  val zero : bignum = {neg=false,coeffs=[]}
  val one : bignum = {neg=false,coeffs=[1]}
  val two : bignum = {neg=false,coeffs=[2]}
  val ten : bignum = {neg=false,coeffs=[10]}

  fun isCanonical ({neg,coeffs}:bignum):bool =
  let
    fun withinRange (x) = (x>=0) andalso (x<1000)
  in
    case (neg,coeffs) of
      (_,[]) => not (neg)
      | _ => (not (List.last (coeffs) = 0) andalso List.all withinRange coeffs)
  end

  fun toInt ({neg,coeffs}:bignum):int option =
    let
      fun loop l =
        (case l
           of [] => 0
            | (x::xs) => x + (1000 * loop (xs)))
    in
      SOME ((if neg then ~1 else 1) * loop (coeffs)) handle _ => NONE
    end

  fun fromInt (n:int):bignum =
    let
      fun loop (n) = if (n=0)
                       then []
                     else (n mod base)::loop (n div base)
    in
      {neg=(n<0), coeffs=loop (Int.abs (n))}
    end

  fun fromList (ns:int list):bignum = 
    {neg=false, coeffs=map Int.abs ns}


end
